import java.util.*;
class Clause {
	ArrayList<Literal> literals;
	int size;
	
	public Clause() {
		literals = new ArrayList<Literal>();
		size = 0;
	}

	public String toString() {
		return this.literals.toString();
	}

	boolean contains(Literal l) {
		for (Literal m : this.literals) {
			if (l.equals(m)) {
				return true;
			}
		}
		return false;
	}

	boolean subset(Clause C) {
		for (Literal l : this.literals) {
			if (!C.contains(l)) {
				return false;
			}
		}
		return true;
	}
	
	boolean equals(Clause C) {
		if (this.subset(C) && C.subset(this)) {
			return true;
		}
		return false;
	}

	boolean isEmpty() {
		if (this.literals.isEmpty()) {
			return true;
		}
		return false;
	}

	void add(Literal l) {
		int n = 1;
		for (Literal m : this.literals) {
			if (l.equals(m)) {
				n = -1;
				break;
			}
		}
		if (n == 1) {
			this.literals.add(l.duplicate());
			this.size++;
		}
	}
	
	void remove(Literal l) {
		for (Literal m : this.literals) {
			if (l.equals(m)) {
				this.literals.remove(m);
				this.size--;
				break;
			}
		}
	}
	
	Literal get(int i) {
		return this.literals.get(i);
	}
	
	Clause duplicate() {
		Clause C = new Clause();
		for (int i = 0; i < this.size; i++) {
			C.add(this.get(i).duplicate());
		}
		return C;
	}
	
	Clause union(Clause C) {
		Clause D = new Clause();
		for (Literal l : this.literals) {
			D.add(l);
		}
		for (Literal l : C.literals) {
			D.add(l);
		}
		return D;
	}
	
	Clause setminus(Clause C) {
		Clause D = new Clause();
		for (Literal l : this.literals) {
			if (!C.contains(l)) {
				D.add(l);
			}
		}
		return D;
	}
	
	Clause substitute(Sbst sbst) {
		Clause C = new Clause();
		for (int i = 0; i < this.size; i++) {
			C.add(this.get(i).substitute(sbst));
		}
		return C;
	}
	
	Clause substituteAtOnce(SbstSet sset) {
		Clause C = new Clause();
		for (int i = 0; i < this.size; i++) {
			C.add(this.get(i).substituteAtOnce(sset));
		}
		return C;
	}
	
	SbstSet variableSeparation(Clause C) {
		SbstSet sset = new SbstSet();
		TermSet tset1 = new TermSet();
		TermSet tset2 = new TermSet();
		for (int i = 0; i < this.size; i++) {
			tset1.add(this.get(i).atom);
		}
		for (int i = 0; i < C.size; i++) {
			tset2.add(C.get(i).atom);
		}
		sset = tset1.variableSeparation(tset2);
		return sset;
	}
	
	ClauseSet resolvents(Clause C) {
		ClauseSet X = new ClauseSet();
		for (int i = 0; i < this.size; i++) {
			Clause C1 = new Clause();
			Clause C2 = new Clause();
			C1 = this.duplicate();
			C2 = C.duplicate();
			SbstSet sset1 = new SbstSet();
			sset1 = C1.variableSeparation(C2);
			C2 = C2.substituteAtOnce(sset1);
			Literal L1 = new Literal();
			L1 = C1.get(i).duplicate();
			for (int j = 0; j < C2.size; j++) {
//			System.out.println("debug print: C2.separated = " + C2);
				Literal L2 = new Literal();
				L2 = C2.get(j).duplicate();
				if (L1.polarity != L2.polarity) {
//					System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") have different polarities.");
					if (L1.atom.unifiable(L2.atom)) {
						Clause C3 = new Clause();
						Clause C4 = new Clause();
						C3 = C1.duplicate();
						C4 = C2.duplicate();
//						System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") are unifiable.");
						SbstSet mgu = new SbstSet();
						mgu = L1.atom.unifier(L2.atom);
//						System.out.println("debug print: mgu(" + i + ", " + j + ") = " + mgu);
						C3.remove(L1);
						C4.remove(L2);
						C3 = C3.substituteAtOnce(mgu);
						C4 = C4.substituteAtOnce(mgu);
						Clause C5 = new Clause();
						C5 = C3.union(C4);
//						System.out.println("debug print: resolvent(" + i + ", " + j + ") = " + C5);
						X.add(C5);
					} else {
//						System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") are not unifiable.");
					}
				} else {
//					System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") have the same polarity.");
				}
			}
		}
		return X;
	}	
	
	ClauseSet resolventsTest(Clause C) {
		ClauseSet X = new ClauseSet();
		for (int i = 0; i < this.size; i++) {
			Clause C1 = new Clause();
			Clause C2 = new Clause();
			C1 = this.duplicate();
			C2 = C.duplicate();
			SbstSet sset1 = new SbstSet();
			sset1 = C1.variableSeparation(C2);
			C2 = C2.substituteAtOnce(sset1);
			Literal L1 = new Literal();
			L1 = C1.get(i).duplicate();
			for (int j = 0; j < C2.size; j++) {
			System.out.println("");
			System.out.println("debug print: C2.separated = " + C2);
				Literal L2 = new Literal();
				L2 = C2.get(j).duplicate();
				if (L1.polarity != L2.polarity) {
					System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") have different polarities.");
					if (L1.atom.unifiable(L2.atom)) {
						Clause C3 = new Clause();
						Clause C4 = new Clause();
						C3 = C1.duplicate();
						C4 = C2.duplicate();
						System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") are unifiable.");
						SbstSet mgu = new SbstSet();
						mgu = L1.atom.unifier(L2.atom);
						System.out.println("debug print: mgu(" + i + ", " + j + ") = " + mgu);
						C3.remove(L1);
						C4.remove(L2);
						C3 = C3.substituteAtOnce(mgu);
						C4 = C4.substituteAtOnce(mgu);
						Clause C5 = new Clause();
						C5 = C3.union(C4);
						System.out.println("debug print: resolvent(" + i + ", " + j + ") = " + C5);
						X.add(C5);
					} else {
						System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") are not unifiable.");
					}
				} else {
					System.out.println("debug print: C1.get(" + i + ") and C2.get(" + j + ") have the same polarity.");
				}
			}
		}
		return X;
	}
	
	public static void main(String[] args) {
		Term P1 = new Term(2, 1, 0);
		Term P2 = new Term(2, 2, 0);
		Term P3 = new Term(2, 3, 0);
		Term P4 = new Term(2, 4, 0);
		Literal L1 = new Literal(1, P1);
		Literal L2 = new Literal(1, P2);
		Literal L3 = new Literal(1, P3);
		Literal L4 = new Literal(1, P4);
		Literal l1 = new Literal(-1, P1);
		Literal l2 = new Literal(-1, P2);
		Literal l3 = new Literal(-1, P3);
		Literal l4 = new Literal(-1, P4);
		Clause C1 = new Clause();
		Clause C2 = new Clause();
		C1.add(L1);
		C1.add(L2);
		C1.add(L3);
		C1.add(L4);
		C1.add(L4);
		C1.add(l1);
		C2.add(l1);
		C2.add(l2);
		C2.add(l3);
		C2.add(l4);
		C2.add(l4);
		C2.add(L1);
		System.out.println(C1);
		System.out.println(C1.size);
		System.out.println(C2);
		System.out.println(C2.size);
		Clause C3 = new Clause();
		C3 = C1.union(C2);
		System.out.println(C3);
		System.out.println(C3.size);
		C1.remove(L4);
		System.out.println(C1);
		System.out.println(C1.size);
		C2.remove(l3);
		System.out.println(C1);
		System.out.println(C1.size);
		System.out.println(C1.variableSeparation(C2));
	}
}
